@@ -9,20 +9,15 @@ work.
 use crate::{
 	cursor::Cursor,
 	domain::*,
-	slice::BitSlice,
-	store::{
+	indices::{
 		BitIdx,
-		BitStore,
+		Indexable,
 	},
+	slice::BitSlice,
+	store::BitStore,
 };
 
 use core::{
-	convert::{
-		AsMut,
-		AsRef,
-		From,
-	},
-	default::Default,
 	fmt::{
 		self,
 		Debug,
@@ -34,27 +29,31 @@ use core::{
 	slice,
 };
 
+#[cfg(any(test, feature = "alloc"))]
+use crate::indices::BitTail;
+
 /// Width in bits of a pointer on the target machine.
 const PTR_BITS: usize = size_of::<*const u8>() * 8;
 
-/// Union to permit reinterpreting a pointer-shaped value as a read pointer,
-/// write pointer, or bare numeric address.
-///
-/// # Safety
-///
-/// Absolutely none whatsoever. This is probably flirting with undefined
-/// behavior, and should be presumed to be the origin site of failure if the
-/// crate ever breaks in the future.
-///
-/// # Type Parameters
-///
-/// - `T`: The referent data type.
-///
-/// # Usage
-///
-/// Don’t.
+/** Union to permit reinterpreting a pointer-shaped value as a read pointer,
+write pointer, or bare numeric address.
+
+# Safety
+
+Absolutely none whatsoever. This is probably flirting with undefined
+behavior, and should be presumed to be the origin site of failure if the
+crate ever breaks in the future.
+
+# Type Parameters
+
+- `T`: The referent data type.
+**/
+#[derive(Clone, Copy)]
 #[doc(hidden)]
-pub(crate) union Pointer<T> {
+pub(crate) union Pointer<T>
+where T: BitStore {
+	/// A shareable pointer to some contended mutable data.
+	a: *const <T as BitStore>::Access,
 	/// A read pointer to some data.
 	r: *const T,
 	/// A write pointer to some data.
@@ -63,7 +62,23 @@ pub(crate) union Pointer<T> {
 	u: usize,
 }
 
-impl<T> Pointer<T> {
+impl<T> Pointer<T>
+where T: BitStore {
+	/// Accesses the address as a shared mutable pointer.
+	///
+	/// # Parameters
+	///
+	/// - `&self`
+	///
+	/// # Returns
+	///
+	/// The stored address, interpreted as a shared pointer to a mutable memory
+	/// location.
+	#[inline]
+	pub(crate) fn a(self) -> *const <T as BitStore>::Access {
+		unsafe { self.a }
+	}
+
 	/// Accesses the address as a read pointer.
 	///
 	/// # Parameters
@@ -74,7 +89,7 @@ impl<T> Pointer<T> {
 	///
 	/// The stored address, as a read pointer.
 	#[inline]
-	pub(crate) fn r(&self) -> *const T {
+	pub(crate) fn r(self) -> *const T {
 		unsafe { self.r }
 	}
 
@@ -88,7 +103,7 @@ impl<T> Pointer<T> {
 	///
 	/// The stored address, as a write pointer.
 	#[inline]
-	pub(crate) fn w(&self) -> *mut T {
+	pub(crate) fn w(self) -> *mut T {
 		unsafe { self.w }
 	}
 
@@ -102,36 +117,41 @@ impl<T> Pointer<T> {
 	///
 	/// The stored address, as a bare integer.
 	#[inline]
-	pub(crate) fn u(&self) -> usize {
+	pub(crate) fn u(self) -> usize {
 		unsafe { self.u }
 	}
 }
 
-impl<T> From<&T> for Pointer<T> {
+impl<T> From<&T> for Pointer<T>
+where T: BitStore {
 	fn from(r: &T) -> Self {
 		Self { r }
 	}
 }
 
-impl<T> From<*const T> for Pointer<T> {
+impl<T> From<*const T> for Pointer<T>
+where T: BitStore {
 	fn from(r: *const T) -> Self {
 		Self { r }
 	}
 }
 
-impl<T> From<&mut T> for Pointer<T> {
+impl<T> From<&mut T> for Pointer<T>
+where T: BitStore {
 	fn from(w: &mut T) -> Self {
 		Self { w }
 	}
 }
 
-impl<T> From<*mut T> for Pointer<T> {
+impl<T> From<*mut T> for Pointer<T>
+where T: BitStore {
 	fn from(w: *mut T) -> Self {
 		Self { w }
 	}
 }
 
-impl<T> From<usize> for Pointer<T> {
+impl<T> From<usize> for Pointer<T>
+where T: BitStore {
 	fn from(u: usize) -> Self {
 		Self { u }
 	}
@@ -273,10 +293,6 @@ where T: BitStore {
 
 impl<T> BitPtr<T>
 where T: BitStore {
-	/// The number of high bits in `self.ptr` that are actually the address of
-	/// the zeroth `T`.
-	pub const PTR_DATA_BITS: usize = PTR_BITS - Self::PTR_HEAD_BITS;
-
 	/// Marks the bits of `self.ptr` that are the `data` section.
 	pub const PTR_DATA_MASK: usize = !Self::PTR_HEAD_MASK;
 
@@ -368,12 +384,11 @@ where T: BitStore {
 	///
 	/// # Parameters
 	///
-	/// - `data: impl Into<Pointer<T>>`: A well-aligned pointer to a storage
-	///   element.
-	/// - `head: impl Into<BitIdx>`: The bit index of the first live bit in the
-	///   element under `*data`.
-	/// - `bits: usize`: The number of live bits in the region the produced
-	///   `BitPtr<T>` describes.
+	/// - `data`: A well-aligned pointer to a storage element.
+	/// - `head`: The bit index of the first live bit in the element under
+	///   `*data`.
+	/// - `bits`: The number of live bits in the region the produced `BitPtr<T>`
+	///   describes.
 	///
 	/// # Returns
 	///
@@ -387,8 +402,6 @@ where T: BitStore {
 	/// This function panics in the following events:
 	///
 	/// - `data` is not well aligned to `T`’s requirements.
-	/// - `head` is not a valid index within `T`, according to
-	/// `BitIdx::is_valid::<T>`.
 	/// - `bits` is larger than `Self::MAX_BITS`.
 	/// - `data` and `bits` describe a `[T]` slice which wraps around the edge
 	///   of the memory space.
@@ -402,10 +415,10 @@ where T: BitStore {
 	/// governable in the caller’s context.
 	pub(crate) fn new(
 		data: impl Into<Pointer<T>>,
-		head: impl Into<BitIdx>,
+		head: BitIdx<T>,
 		bits: usize,
 	) -> Self {
-		let (data, head) = (data.into(), head.into());
+		let data = data.into();
 
 		//  Null pointers become the empty slice.
 		if data.r().is_null() {
@@ -427,14 +440,7 @@ where T: BitStore {
 			Self::MAX_BITS,
 		);
 
-		assert!(
-			head.is_valid::<T>(),
-			"BitPtr head index cannot be {}; the valid domain is 0 .. {}",
-			head,
-			T::BITS,
-		);
-
-		let elts = head.span::<T>(bits).0;
+		let elts = head.span(bits).0;
 		let tail = data.r().wrapping_add(elts);
 		assert!(
 			tail >= data.r(),
@@ -469,10 +475,10 @@ where T: BitStore {
 	/// [`::new`]: #method.new
 	pub(crate) unsafe fn new_unchecked(
 		data: impl Into<Pointer<T>>,
-		head: impl Into<BitIdx>,
+		head: BitIdx<T>,
 		bits: usize,
 	) -> Self {
-		let (data, head) = (data.into(), *head.into() as usize);
+		let (data, head) = (data.into(), *head as usize);
 
 		let ptr_data = data.u() & Self::PTR_DATA_MASK;
 		let ptr_head = head >> Self::LEN_HEAD_BITS;
@@ -545,11 +551,11 @@ where T: BitStore {
 	/// A `BitIdx` that is the index of the first live bit in the first element.
 	/// This will be in the domain `0 .. T::BITS`.
 	#[inline]
-	pub fn head(&self) -> BitIdx {
+	pub fn head(&self) -> BitIdx<T> {
 		let ptr = self.ptr.as_ptr() as usize;
 		let ptr_head = (ptr & Self::PTR_HEAD_MASK) << Self::LEN_HEAD_BITS;
 		let len_head = self.len & Self::LEN_HEAD_MASK;
-		((ptr_head | len_head) as u8).into()
+		((ptr_head | len_head) as u8).idx()
 	}
 
 	/// Counts how many bits are in the domain of a `BitPtr` slice.
@@ -577,6 +583,7 @@ where T: BitStore {
 	/// # Safety
 	///
 	/// None. The caller must ensure that the invariants of `::new` are upheld.
+	#[cfg(feature = "alloc")]
 	#[inline]
 	pub unsafe fn set_len(&mut self, len: usize) {
 		let n = (len << Self::LEN_HEAD_BITS) | (self.len & Self::LEN_HEAD_MASK);
@@ -596,7 +603,7 @@ where T: BitStore {
 	/// - `.1: BitIdx`: The index of the first live bit in the bit region.
 	/// - `.2: usize`: The number of live bits in the bit region.
 	#[inline]
-	pub(crate) fn raw_parts(&self) -> (Pointer<T>, BitIdx, usize) {
+	pub(crate) fn raw_parts(&self) -> (Pointer<T>, BitIdx<T>, usize) {
 		(self.pointer(), self.head(), self.len())
 	}
 
@@ -615,7 +622,7 @@ where T: BitStore {
 	/// This size must be valid in the user’s memory model and allocation
 	/// regime.
 	pub fn elements(&self) -> usize {
-		self.head().span::<T>(self.len()).0
+		self.head().span(self.len()).0
 	}
 
 	/// Extracts the element cursor of the first dead bit *after* the tail bit.
@@ -626,30 +633,26 @@ where T: BitStore {
 	///
 	/// # Returns
 	///
-	/// A `BitIdx` that is the index of the first dead bit after the last live
-	/// bit in the last element. This will be in the domain `1 ..= T::BITS`.
+	/// A `BitTail` that is the index of the first dead bit after the last live
+	/// bit in the last element. This will almost always be in the domain
+	/// `1 ..= T::BITS`.
+	#[cfg(any(test, feature = "alloc"))]
 	#[inline]
-	pub fn tail(&self) -> BitIdx {
-		/*
-		 * This function is one of the most-used in the library. As such, its
-		 * implementation is written in a straight linear style. The compiler is
-		 * free to rearrange the code as it sees fit, and may not reflect the
-		 * code below. The equivalent, user-friendly function body is:
-		 *
-		 * if self.is_empty() {
-		 * 	return 0.into();
-		 * }
-		 * let bits = (self.len & Self::LEN_TAIL_MASK) >> Self::LEN_HEAD_BITS;
-		 * if bits == 0 { T::BITS } else { bits as u8 }.into()
-		 */
+	pub(crate) fn tail(&self) -> BitTail<T> {
+		let (head, len) = (self.head(), self.len());
 
-		//  Compute the in-element tail index as the head plus the length, with
-		//  the modulus of the element width.
-		let tail = (*self.head() as usize + self.len()) & T::MASK as usize;
-		//  If the tail’s bit pattern is zero, wrap it to the maximal. This
-		//  upshifts `1` (pattern is zero) or `0` (pattern is not), then
-		//  sets the upshift bit on the pattern.
-		((((tail == 0) as u8) << T::INDX) | tail as u8).into()
+		if *head == 0 && len == 0 {
+			return 0u8.tail();
+		}
+
+		//  Compute the in-element tail index as the head plus the length,
+		//  modulated to the element width.
+		let tail = (*self.head() as usize + len) & T::MASK as usize;
+		//  If the tail is zero, wrap it to `T::BITS` as the maximal. This
+		//  upshifts `1` (tail is zero) or `0` (tail is not), then sets the
+		//  upshift on the rest of the tail, producing something in the range
+		//  `1 ..= T::BITS`.
+		((((tail == 0) as u8) << T::INDX) | tail as u8).tail()
 	}
 
 	/// Checks if the pointer represents the empty slice.
@@ -663,27 +666,12 @@ where T: BitStore {
 	/// # Returns
 	///
 	/// Whether the slice is empty or populated.
+	#[cfg(any(test, feature = "alloc"))]
 	#[inline]
 	pub fn is_empty(&self) -> bool {
 		self.len & !Self::LEN_HEAD_MASK == 0
 	}
 
-	/// Checks if the pointer represents the full slice.
-	///
-	/// The full slice has `!0` `bits` counters.
-	///
-	/// # Parameters
-	///
-	/// - `&self`
-	///
-	/// # Returns
-	///
-	/// Whether the slice is full or has room for more growth.
-	#[inline]
-	pub fn is_full(&self) -> bool {
-		self.len | Self::LEN_HEAD_MASK == !0
-	}
-
 	/// Accesses the element slice behind the pointer as a Rust slice.
 	///
 	/// # Parameters
@@ -718,7 +706,7 @@ where T: BitStore {
 		unsafe { slice::from_raw_parts_mut(self.pointer().w, self.elements()) }
 	}
 
-	/// Gets the domain kind for the region the pointer describes.
+	/// Accesses the element slice behind the pointer as a shared-mutable slice.
 	///
 	/// # Parameters
 	///
@@ -726,39 +714,29 @@ where T: BitStore {
 	///
 	/// # Returns
 	///
-	/// An enum describing the live bits in the region the pointer covers.
-	pub fn domain_kind(&self) -> BitDomainKind {
+	/// Standard Rust slice handle over the data governed by this pointer.
+	///
+	/// # Lifetimes
+	///
+	/// - `'a`: Lifetime for which the data behind the pointer is live.
+	pub fn as_access_slice<'a>(&self) -> &'a [T::Access] {
+		unsafe { slice::from_raw_parts(self.pointer().a, self.elements()) }
+	}
+
+	/// Gets the domain for the region the pointer describes.
+	///
+	/// # Parameters
+	///
+	/// - `self`
+	///
+	/// # Returns
+	///
+	/// An enum containing the logical components of the domain governed by
+	/// `self`.
+	pub(crate) fn domain<'a>(self) -> BitDomain<'a, T> {
 		self.into()
 	}
 
-	/// Gets the domain for the region the pointer describes.
-	///
-	/// # Parameters
-	///
-	/// - `&self`
-	///
-	/// # Returns
-	///
-	/// An enum containing the logical components of the domain governed by
-	/// `self`.
-	pub fn domain(&self) -> BitDomain<T> {
-		(*self).into()
-	}
-
-	/// Gets the domain for the region the pointer describes.
-	///
-	/// # Parameters
-	///
-	/// - `&self`
-	///
-	/// # Returns
-	///
-	/// An enum containing the logical components of the domain governed by
-	/// `self`.
-	pub fn domain_mut(&self) -> BitDomainMut<T> {
-		(*self).into()
-	}
-
 	/// Moves the `head` cursor upwards by one.
 	///
 	/// If `head` is at the back edge of the first element, then it will be set
@@ -775,12 +753,13 @@ where T: BitStore {
 	/// memory. It mutates the pointer and element count, so if this pointer is
 	/// solely responsible for owned memory, its conception of the allocation
 	/// will differ from the allocator’s.
+	#[cfg(feature = "alloc")]
 	pub unsafe fn incr_head(&mut self) {
 		let (data, head, bits) = self.raw_parts();
 		if bits == 0 {
 			return;
 		}
-		let (head, wrap) = head.incr::<T>();
+		let (head, wrap) = head.incr();
 		*self = Self::new_unchecked(
 			data.r().offset(wrap as isize),
 			head,
@@ -804,12 +783,13 @@ where T: BitStore {
 	/// memory. It mutates the pointer and element count, so if this pointer is
 	/// solely responsible for owned memory, its conception of the allocation
 	/// will differ from the allocator’s.
+	#[cfg(feature = "alloc")]
 	pub unsafe fn decr_head(&mut self) {
 		let (data, head, bits) = self.raw_parts();
 		if bits == 0 {
 			return;
 		}
-		let (head, wrap) = head.decr::<T>();
+		let (head, wrap) = head.decr();
 		*self = Self::new_unchecked(
 			data.r().offset(-(wrap as isize)),
 			head,
@@ -833,6 +813,7 @@ where T: BitStore {
 	/// memory. It mutates the element count, so if this pointer is solely
 	/// responsible for owned memory, its conception of the allocation will
 	/// differ from the allocator’s.
+	#[cfg(feature = "alloc")]
 	pub unsafe fn incr_tail(&mut self) {
 		let len = self.len();
 		if len == Self::MAX_BITS {
@@ -857,6 +838,7 @@ where T: BitStore {
 	/// memory. It mutates the element count, so if this pointer is solely
 	/// responsible for owned memory, its conception of the allocation will
 	/// differ from the allocator’s.
+	#[cfg(feature = "alloc")]
 	pub unsafe fn decr_tail(&mut self) {
 		let len = self.len();
 		if len == 0 {
@@ -925,14 +907,15 @@ where T: BitStore {
 	}
 }
 
-/// Gets write access to all elements in the underlying storage, including the
-/// partial head and tail elements.
-///
-/// # Safety
-///
-/// This is *unsafe* to use except from known mutable `BitSlice` structures.
-/// Mutability is not encoded in the `BitPtr` type system at this time, and thus
-/// is not enforced by the compiler yet.
+/** Gets write access to all elements in the underlying storage, including the
+partial head and tail elements.
+
+# Safety
+
+This is *unsafe* to use except from known mutable `BitSlice` structures.
+Mutability is not encoded in the `BitPtr` type system at this time, and thus is
+not enforced by the compiler yet.
+**/
 impl<T> AsMut<[T]> for BitPtr<T>
 where T: BitStore {
 	fn as_mut(&mut self) -> &mut [T] {
@@ -987,8 +970,8 @@ where T: BitStore {
 			}
 		}
 
-		struct BinAddr<T: BitStore>(BitIdx, PhantomData<T>);
-		impl<T: BitStore>  Debug for BinAddr<T> {
+		struct BinAddr<T: BitStore>(BitIdx<T>);
+		impl<T: BitStore> Debug for BinAddr<T> {
 			fn fmt(&self, f: &mut Formatter) -> fmt::Result {
 				write!(f, "0b{:0>1$b}", *self.0, T::INDX as usize)
 			}
@@ -997,7 +980,7 @@ where T: BitStore {
 		write!(f, "BitPtr<{}>", T::TYPENAME)?;
 		f.debug_struct("")
 			.field("data", &HexPtr::<T>(self.pointer().r()))
-			.field("head", &BinAddr::<T>(self.head(), PhantomData))
+			.field("head", &BinAddr::<T>(self.head()))
 			.field("bits", &self.len())
 			.finish()
 	}
@@ -1009,7 +992,6 @@ mod tests {
 
 	#[test]
 	fn associated_consts_u8() {
-		assert_eq!(BitPtr::<u8>::PTR_DATA_BITS, PTR_BITS);
 		assert_eq!(BitPtr::<u8>::PTR_HEAD_BITS, 0);
 
 		assert_eq!(BitPtr::<u8>::PTR_DATA_MASK, !0);
@@ -1018,7 +1000,6 @@ mod tests {
 
 	#[test]
 	fn associated_consts_u16() {
-		assert_eq!(BitPtr::<u16>::PTR_DATA_BITS, PTR_BITS - 1);
 		assert_eq!(BitPtr::<u16>::PTR_HEAD_BITS, 1);
 
 		assert_eq!(BitPtr::<u16>::PTR_DATA_MASK, !0 << 1);
@@ -1027,7 +1008,6 @@ mod tests {
 
 	#[test]
 	fn associated_consts_u32() {
-		assert_eq!(BitPtr::<u32>::PTR_DATA_BITS, PTR_BITS - 2);
 		assert_eq!(BitPtr::<u32>::PTR_HEAD_BITS, 2);
 
 		assert_eq!(BitPtr::<u32>::PTR_DATA_MASK, !0 << 2);
@@ -1036,7 +1016,6 @@ mod tests {
 
 	#[test]
 	fn associated_consts_u64() {
-		assert_eq!(BitPtr::<u64>::PTR_DATA_BITS, PTR_BITS - 3);
 		assert_eq!(BitPtr::<u64>::PTR_HEAD_BITS, 3);
 
 		assert_eq!(BitPtr::<u64>::PTR_DATA_MASK, !0 << 3);
@@ -1045,8 +1024,8 @@ mod tests {
 
 	#[test]
 	fn ctors() {
-		let data: [u32; 4] = [0x756c6153, 0x2c6e6f74, 0x6e6f6d20, 0x00216f64];
-		let bp = BitPtr::<u32>::new(&data as *const u32, 0, 32 * 4);
+		let data: [u32; 4] = [0; 4];
+		let bp = BitPtr::<u32>::new(&data as *const u32, 0u8.idx(), 32 * 4);
 		assert_eq!(bp.pointer().r(), &data as *const u32);
 		assert_eq!(bp.elements(), 4);
 		assert_eq!(*bp.head(), 0);
@@ -1057,16 +1036,17 @@ mod tests {
 	fn empty() {
 		let data = [0u8; 4];
 		//  anything with 0 bits is unconditionally empty
-		let bp = BitPtr::<u8>::new(&data as *const u8, 2, 0);
+		let bp = BitPtr::<u8>::new(&data as *const u8, 2u8.idx(), 0);
 
 		assert!(bp.is_empty());
 		assert_eq!(*bp.head(), 2);
 		assert_eq!(*bp.tail(), 2);
 	}
 
+	#[cfg(not(miri))]
 	#[test]
 	#[should_panic]
 	fn overfull() {
-		BitPtr::<u32>::new(8 as *const u32, 1, BitPtr::<u32>::MAX_BITS + 1);
+		BitPtr::<u32>::new(8 as *const u32, 1u8.idx(), BitPtr::<u32>::MAX_BITS + 1);
 	}
 }
