@@ -25,6 +25,7 @@ use crate::{
 #[cfg(feature = "alloc")]
 use core::{
 	cmp,
+	convert::TryInto,
 	fmt::{
 		self,
 		Formatter,
@@ -47,8 +48,10 @@ use serde::{
 	de::{
 		self,
 		Deserializer,
+		Error,
 		MapAccess,
 		SeqAccess,
+		Unexpected,
 		Visitor,
 	},
 };
@@ -90,7 +93,14 @@ where C: Cursor, T: BitStore + Deserialize<'de> {
 		let data: Box<[T]> = seq.next_element()?
 			.ok_or_else(|| de::Error::invalid_length(2, &self))?;
 
-		let bitptr = BitPtr::new(data.as_ptr(), head, bits);
+		let bitptr = BitPtr::new(
+			data.as_ptr(),
+			head.try_into().map_err(|_| Error::invalid_value(
+				Unexpected::Unsigned(u64::from(head)),
+				&self,
+			))?,
+			bits,
+		);
 		mem::forget(data);
 		Ok(unsafe { BitBox::from_raw(bitptr) })
 	}
@@ -125,7 +135,10 @@ where C: Cursor, T: BitStore + Deserialize<'de> {
 
 		let bitptr = BitPtr::new(
 			data.as_ptr(),
-			head,
+			head.try_into().map_err(|_| Error::invalid_value(
+				Unexpected::Unsigned(u64::from(head)),
+				&self,
+			))?,
 			cmp::min(bits, data.len() * T::BITS as usize),
 		);
 		mem::forget(data);
@@ -157,7 +170,7 @@ where C: Cursor, T: 'de + BitStore + Deserialize<'de> {
 }
 
 impl<C, T> Serialize for BitSlice<C, T>
-where C: Cursor, T: BitStore + Serialize {
+where C: Cursor, T: BitStore + Serialize, T::Access: Serialize {
 	fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
 	where S: Serializer {
 		let head = self.bitptr().head();
@@ -165,7 +178,7 @@ where C: Cursor, T: BitStore + Serialize {
 
 		state.serialize_field("head", &*head)?;
 		state.serialize_field("bits", &(self.len() as u64))?;
-		state.serialize_field("data", self.as_slice())?;
+		state.serialize_field("data", self.as_total_slice())?;
 
 		state.end()
 	}
@@ -173,7 +186,7 @@ where C: Cursor, T: BitStore + Serialize {
 
 #[cfg(feature = "alloc")]
 impl<C, T> Serialize for BitBox<C, T>
-where C: Cursor, T: BitStore + Serialize {
+where C: Cursor, T: BitStore + Serialize, T::Access: Serialize {
 	fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
 	where S: Serializer {
 		BitSlice::serialize(&*self, serializer)
@@ -182,7 +195,7 @@ where C: Cursor, T: BitStore + Serialize {
 
 #[cfg(feature = "alloc")]
 impl<C, T> Serialize for BitVec<C, T>
-where C: Cursor, T: BitStore + Serialize {
+where C: Cursor, T: BitStore + Serialize, T::Access: Serialize {
 	fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
 	where S: Serializer {
 		BitSlice::serialize(&*self, serializer)
@@ -237,7 +250,7 @@ mod tests {
 	#[cfg(feature = "alloc")]
 	#[test]
 	fn small() {
-		let bv = bitvec![1; 5];
+		let bv = bitvec![BigEndian, u8; 1; 5];
 		let bs = &bv[1 ..];
 		assert_ser_tokens(&bs, bvtok![s 1, 1, 4, U8, 0b1111_1000]);
 
@@ -252,15 +265,14 @@ mod tests {
 	#[test]
 	fn wide() {
 		let src: &[u8] = &[0, !0];
-		let bs: &BitSlice = src.into();
+		let bs = src.bits::<Local>();
 		assert_ser_tokens(&(&bs[1 .. 15]), bvtok![s 2, 1, 14, U8, 0, !0]);
 	}
 
 	#[cfg(feature = "alloc")]
 	#[test]
-	#[cfg(feature = "alloc")]
 	fn deser() {
-		let bv = bitvec![0, 1, 1, 0, 1, 0];
+		let bv = bitvec![BigEndian, u8; 0, 1, 1, 0, 1, 0];
 		assert_de_tokens(&bv, bvtok![d 1, 0, 6, U8, 0b0110_1000]);
 		//  test that the bits outside the bits domain don't matter in deser
 		assert_de_tokens(&bv, bvtok![d 1, 0, 6, U8, 0b0110_1001]);
